#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#   makeRunTopoSet
#
#       topoSetDictを作り、topoSetを実行する
#       カレントcaseより実行
#
#       ＜使い方＞
#       makeSetOrZone <caseDir>
#           caseDir     :case directory
#
#       12/09/25    新規作成
#          12/16    makeCombiContentsを追加
#       13/01/22    setSourceName, setSourceNameWqを修正（「;」が取得できない）
#          01/27    regionToCellを修正
#          02/03    cellToFace:cellSet(Zone)からcell表面のfaceSet(Zone)を抽出を追加
#          08/11    setSourceName:nameがstrかどうかをチェック。strでない場合(list)は、
#                   空文を返す。
#


import sys
import os
import pyTreeFoam
import getFoamContents

#-----------sourceの取得---------------
#cellSet----
def cellToCell(name):
    cont = """    // Copy elements from cellSet
    source cellToCell;
    sourceInfo
    {
        set c1;
    }"""
    cont = setSourceName(cont, "set", name)
    return cont

def faceToCell(name):
    cont ="""    // Select based on faceSet
    source faceToCell;
    sourceInfo
    {
        set f0;             // Name of faceSet

        //option neighbour; // cell with neighbour in faceSet
        //option owner;     //  ,,       owner
        option any;         // cell with any face in faceSet
        //option all;       // cell with all faces in faceSet
    }"""
    cont = setSourceName(cont, "set", name)
    return cont

def pointToCell(name):
    cont = """    // Select based on pointSet
    source pointToCell;
    sourceInfo
    {
        set p0;
        option any;         // cell with any point in pointSet
        //option all;       // cell with all points in pointSet
    }"""
    cont = setSourceName(cont, "set", name)
    return cont

def zoneToCell(name):
    cont = """    // Cells in cell zone
    source zoneToCell;
    sourceInfo
    {
        name ".*Zone";      // Name of cellZone, regular expressions allowed
    }"""
    cont = setSourceNameWq(cont, "name", name)
    return cont

def faceZoneToCell(name):
    cont = """    // Cells on master or slave side of faceZone
    source faceZoneToCell;
    sourceInfo
    {
        name ".*Zone";      // Name of faceZone, regular expressions allowed
        option master;      // master/slave
    }"""
    cont = setSourceNameWq(cont, "name", name)
    return cont

def surfaceToCell(name):
    cont = """    // Select based on surface
    source surfaceToCell;
    sourceInfo
    {
        file            "www.avl.com-geometry.stl";
        outsidePoints   ((-99 -99 -59));    // definition of outside
        includeCut      false;              // cells cut by surface
        includeInside   false;              // cells not on outside of surf
        includeOutside  false;              // cells on outside of surf
        nearDistance    -1;                 // cells with centre near surf
                                            // (set to -1 if not used)
        curvature       0.9;                // cells within nearDistance
                                            // and near surf curvature
                                            // (set to -100 if not used)
    }"""
    cont = setSourceNameWq(cont, "file", name)
    return cont

def labelToCell(name):
    cont = """    // Select by explicitly providing cell labels
    source labelToCell;
    sourceInfo
    {
        value (12 13 56);   // labels of cells
    }"""
    return cont

def shapeToCell(name):
    cont = """    // Select based on cellShape
    source shapeToCell;
    sourceInfo
    {
        type hex;           // hex/wedge/prism/pyr/tet/tetWedge/splitHex
    }"""
    return cont

def boxToCell(name):
    cont ="""    // Cells with cell centre within box
    source boxToCell;
    sourceInfo
    {
       box   (0 0 0) (1 1 1);
    }"""
    return cont

def cylinderToCell(name):
    cont = """    // Cells with centre within cylinder
    source cylinderToCell;
    sourceInfo
    {
       p1       (0.2 0.2 -10); // start point on cylinder axis
       p2       (0.2 0.2 0);   // end point on cylinder axis
       radius   5.0;
    }"""
    return cont

def sphereToCell(name):
    cont = """    // Cells with centre within sphere
    source sphereToCell;
    sourceInfo
    {
       centre   (0.2 0.2 -10);
       radius   5.0;
    }"""
    return cont

def rotatedBoxToCell(name):
    cont = """    // Cells with cell centre within box
    // Is skewed, rotated box. Given as origin and three spanning vectors.
    source rotatedBoxToCell;
    sourceInfo
    {
       origin   (0.2 0.2 -10);
       i        (0.2 0.2 0);
       j        (-0.2 0.2 0);
       k        (10 10 10);
    }"""
    return cont

def nearestToCell(name):
    cont = """    // Cells with cellCentre nearest to coordinates
    source nearestToCell;
    sourceInfo
    {
       points ((0 0 0) (1 1 1)(2 2 2));
    }"""
    return cont

def fieldToCell(name):
    cont = """    // values of field within certain range
    source fieldToCell;
    sourceInfo
    {
        fieldName   U;      // Note: uses mag(U) since volVectorField
        min         0.1;
        max         0.5;
    }"""
    return cont

def regionToCell(name):
    cont = """    // Mesh region (non-face connected part of (subset of)mesh)
    source regionToCell;
    sourceInfo
    {
        set         c0;         // name of cellSet giving mesh subset
        insidePoint (1 2 3);    // point inside region to select
    }"""
    #cont = setSourceNameWq(cont, "set", name)
    cont = setSourceName(cont, "set", name)
    return cont

#faceset----
def cellToFace(name):
    cont = """    // Select based on cellSet
    source cellToFace;
    sourceInfo
    {
        set c0;
        option all;         // All faces of cells
        //option both;      // Only faces whose owner&neighbour are in cellSet
    }"""
    cont = setSourceName(cont, "set", name)
    return cont

def faceToFace(name):
    cont = """    // Copy elements from faceSet
    source faceToFace;
    sourceInfo
    {
        set f1;
    }"""
    cont = setSourceName(cont, "set", name)
    return cont

def pointToFace(name):
    cont = """    // Select based on pointSet
    source pointToFace;
    sourceInfo
    {
        set p0;
        //option any;     // Faces using any point in pointSet
        option all;       // Faces with all points in pointSet
    }"""
    cont = setSourceName(cont, "set", name)
    return cont

def zoneToFace(name):
    cont = """    // All faces of faceZone
    source zoneToFace;
    sourceInfo
    {
        name ".*Zone1";     // Name of faceZone, regular expressions allowed
    }"""
    cont = setSourceNameWq(cont, "name", name)
    return cont

def labelToFace(name):
    cont = """    //  Select by explicitly providing face labels
    source labelToFace;
    sourceInfo
    {
        value (12 13 56);   // labels of faces
    }"""
    return cont

def boxToFace(name):
    cont = """    // Faces with face centre within box
    source boxToFace;
    sourceInfo
    {
        box  (0 0 0) (1 1 1);
    }"""
    return cont

def patchToFace(name):
    cont = """    // All faces of patch
    source patchToFace;
    sourceInfo
    {
        name ".*Wall";      // Name of patch, regular expressions allowed
    }"""
    cont = setSourceNameWq(cont, "name", name)
    return cont

def normalToFace(name):
    cont = """    // Faces with normal to within certain angle aligned with vector.
    source normalToFace;
    sourceInfo
    {
        normal (0 0 1);     // Vector
        cos     0.01;       // Tolerance (max cos of angle)
    }"""
    return cont

#pointSet----
def cellToPoint(name):
    cont = """    // Select based on cellSet
    source cellToPoint;
    sourceInfo
    {
        set c0;
        option all;         // all points of cell
    }"""
    cont = setSourceName(cont, "set", name)
    return cont

def faceToPoint(name):
    cont = """    // Select based on faceSet
    source faceToPoint;
    sourceInfo
    {
        set f0;             // name of faceSet
        option all;         // all points of face
    }"""
    cont = setSourceName(cont, "set", name)
    return cont

def pointToPoint(name):
    cont = """    // Copy elements from pointSet
    source pointToPoint;
    sourceInfo
    {
        set p1;
    }"""
    cont = setSourceName(cont, "set", name)
    return cont
    
def zoneToPoint(name):
    cont = """    // All points in pointzone
    source zoneToPoint;
    sourceInfo
    {
        name ".*Zone";      // name of pointZone, regular expressions allowed
    }"""
    cont = setSourceNameWq(cont, "name", name)
    return cont

def surfaceToPoint(name):
    cont = """    // Select based on surface
    source surfaceToPoint;
    sourceInfo
    {
        file            "www.avl.com-geometry.stl";
        nearDistance    0.1;    // points near to surface
        includeInside   false;  // points on inside of surface
                                // (requires closed surface with consistent
                                //  normals)
        includeOutside  false;  //   ,,    outside  ,,
    }"""
    cont = setSourceNameWq(cont, "file", name)
    return cont

def labelToPoint(name):
    cont = """    // Select by explicitly providing point labels
    source labelToPoint;
    sourceInfo
    {
        value (12 13 56);   // labels of points
    }"""
    return cont

def boxToPoint(name):
    cont = """    // Points with coordinate within box
    source boxToPoint;
    sourceInfo
    {
       box   (0 0 0) (1 1 1);
    }"""
    return cont

def nearestToPoint(name):
    cont = """    // Points nearest to coordinates
    source nearestToPoint;
    sourceInfo
    {
       points ((0 0 0) (1 1 1));
    }"""
    return cont

#cellZone----
def setToCellZone(name):
    cont = """    // Select based on cellSet
    source setToCellZone;
    sourceInfo
    {
        set c0;           // name of cellSet
    }"""
    cont = setSourceName(cont, "set", name)
    return cont

#faceZone----
def setToFaceZone(name):
    cont = """    // Select based on faceSet without orientation
    source setToFaceZone;
    sourceInfo
    {
        faceSet f0;       // name of faceSet
    }"""
    cont = setSourceName(cont, "faceSet", name)
    return cont

def setsToFaceZone(names):
    cont = """    // Select based on faceSet, using cellSet to determine orientation
    source setsToFaceZone;
    sourceInfo
    {
        faceSet f0;       // name of faceSet
        cellSet c0;       // name of cellSet of (slave?) master side
    }"""
    for name in names:
        setName = name.split(":")
        if len(setName) == 2:
            if setName[0] == "c":
                cont = setSourceName(cont, "cellSet", setName[1])
            elif setName[0] == "f":
                cont = setSourceName(cont, "faceSet", setName[1])
    return cont

#-----------------------

#  setOption
#    optionの内容を設定
def setOption(cont, optionCont):
    newCont = setSourceName(cont, "option", optionCont)
    return newCont

def setSourceName(cont, comm, name):
    if type(name) == list:
        return ""

    lines = cont.split("\n")
    i=0
    for line in lines:
        [keyword, p] = getFoamContents.getKeywordMem(line, 0)
        if keyword == comm:
            sp = p
            [keyword, ep] = getFoamContents.getKeywordMem(line, sp)
            #次の1行を追加（";"が取得出来ないため）13/01/22
            ep = ep-1
            #次のif文を削除　12/01/22
            #if ep < len(line):
            #    ep = ep-1
            #newline = line[:p] + " " + name + line[ep-1:]
            newline = line[:p] + " " + name + line[ep:]
            lines[i] = newline
            break
        i+=1
    newCont = "\n".join(lines)
    return newCont

#---------------------
def setSourceNameWq(cont, comm, name):
    lines = cont.split("\n")
    i=0
    for line in lines:
        [keyword, p] = getFoamContents.getKeywordMem(line, 0)
        if keyword == comm:
            sp = p
            [keyword, ep] = getFoamContents.getKeywordMem(line, sp)
            ep = ep-1
            #if ep < len(line):
            #    ep = ep-1
            #newline = line[:p] + ' "' + name + '"' + line[ep-1:]
            newline = line[:p] + ' "' + name + '"' + line[ep:]
            lines[i] = newline
            break
        i+=1
    newCont = "\n".join(lines)
    return newCont

def add4Spaces(cont):
    lines = cont.split("\n")
    i=0
    for line in lines:
        lines[i] = "    " + line
        i+=1
    cont = "\n".join(lines)
    return cont

#
#  makeCombiContents
#    組み合わせActionの内容を作成
def makeCombiContents(sourceType, sourceName, action, resultType, resultName):

    #rename　SetとZoneの名称変更
    if action == "renameSetZone":
        if sourceType.find("Zone") >= 0:
            if sourceType == "cellZone":
                #cont  = makeContents(sourceType, sourceName, "new",
                #        "cellSet", resultName)
                cont  = makeContents(sourceType, sourceName, "new",
                        "cellSet", "temp")
                #cont += makeContents("cellSet", resultName, "new",
                #        "cellZoneSet", resultName)
                cont += makeContents("cellSet", "temp", "new",
                        "cellZoneSet", resultName)
                cont += makeContents("cellSet", sourceName, "remove",
                        "cellZoneSet", sourceName)
                cont += makeContents("cellSet", "temp", "remove",
                        "cellZoneSet", "temp")
            else:
                cont  = makeContents(sourceType, sourceName, "new",
                        "faceSet", resultName)
                cont += makeContents("faceSet", sourceName, "new",
                        "faceZoneSet", resultName)
                cont += makeContents("faceSet", sourceName, "remove",
                        "faceZoneSet", sourceName)
        elif sourceType.find("Set") >= 0:
            cont  = makeContents(sourceType, sourceName, "new",
                        sourceType, resultName)
            cont += makeContents(sourceType, sourceName, "remove",
                        sourceType, sourceName)
        else:
            cont = u""
        return cont

    #newAdds　cellSetを複数加え、新しい名称をつける
    elif action == "newAddsSet":
        if sourceType != "cellSet":
            return ""
        if type(sourceName) == str:
            #one sourceName
            cont  = makeContents(sourceType, sourceName, "new",
                        "cellSet", resultName)
            return cont
        else:
            #multi sourceName
            cont  = makeContents(sourceType, sourceName[0], "new",
                        "cellSet", resultName)
            i=1
            while i<len(sourceName):
                cont += makeContents(sourceType, sourceName[i], "add",
                        "cellSet", resultName)
                i+=1
        return cont

    #newInvert　cellSetの反転をnewNameでつくる
    elif action == "newInvertSet":
        if sourceType == "cellSet":
            cont  = makeContents(sourceType, sourceName, "new",
                        "cellSet", resultName)
            cont += makeContents(sourceType, resultName, "invert",
                        "cellSet", resultName)
        else:
            cont - ""
        return cont

    #newCellToFace  cellSet（Zone）からcell表面のfaceSet（Zone）を作る
    elif action == "newCellToFace":
        if sourceType == "cellSet":
            cont = makeContentsCellToFace(sourceType, sourceName, action,
                    resultType, resultName)
        elif sourceType == "cellZone":
            cont = makeContents(sourceType, sourceName, "new", "cellSet",
                    sourceName)
            cont += makeContentsCellToFace("cellSet", sourceName, action,
                    resultType, resultName)
            cont += makeContents("faceSet", resultName, "new", "faceZoneSet",
                    resultName)
        return cont
    #newZonesToSets:cellZone(faceZone)からcellSet(faceSet)を
    elif action == "newZonesToSets":
        if sourceType == "cellZone":
            if type(sourceName) != list:
                #one sourceName(string)
                cont = makeContents(sourceType, sourceName, "new", "cellSet",
                        sourceName)
            else:
                #multi sourceName(list)
                cont = ""
                i = 0
                while i<len(sourceName):
                    cont += makeContents(sourceType, sourceName[i], "new", "cellSet",
                            sourceName[i])
                    i += 1
            return cont
        elif sourceType == "faceZone":
            if type(sourceName) != list:
                #one sourceName(string)
                cont = makeContents(sourceType, sourceName, "new", "faceSet",
                        sourceName)
            else:
                #multi sourceName(list)
                cont = ""
                i = 0
                while i<len(sourceName):
                    cont += makeContents(sourceType, sourceName[i], "new", "faceSet",
                            sourceName[i])
                    i += 1
            return cont
    else:
        return ""


def makeContentsCellToFace(sourceType, sourceName, action, resultType, resultName):
    Tcont  = makeContents(sourceType, sourceName, "new", "faceSet", "temp")
    cont = setOption(Tcont, "both")
    Tcont = makeContents(sourceType, sourceName, "new", "faceSet", resultName)
    cont += setOption(Tcont, "all")
    cont += makeContents("faceSet", "temp", "delete", "faceSet", resultName)
    cont += makeContents(sourceType, sourceName, "remove", "faceSet", "temp")
    return cont


#  makeContents
#    処理内容を作成
def makeContents(sourceType, sourceName, action, resultType, resultName):
    contU = makeContentsHeader(action, sourceType, resultType, resultName)
    cont = getSourceInfo(sourceType, sourceName, action, resultType)
    if cont != "":
        cont = add4Spaces(cont)
    footer = makeContentsFooter()
    contents = contU + cont + footer
    return contents

#  makeContentsHeader
#    処理内容のheader部作成
def makeContentsHeader(action, sourceType, resultType, resultName):
    contU  = "\n    // " + action + " To " + resultType + "\n"
    contU += "    {\n"
    contU += "        name    " + resultName + ";\n"
    contU += "        type    " + resultType + ";\n"
    contU += "        action  " + action + ";\n"
    return contU    

#  makeContentsFooter
#    処理のfooterを作成
def makeContentsFooter():
    cont = "\n    }\n"
    return cont

#  getSourceInfo
#    sourceのoptionを取得
def getSourceInfo(sourceType, sourceName, action, resultType):
    cont = ""
    if action == "clear" or action == "invert" or action == "remove":
        return cont

    if resultType == "cellSet":
        if   sourceType == "cellSet":
            cont = cellToCell(sourceName)
        elif sourceType == "faceSet":
            cont = faceToCell(sourceName)
        elif sourceType == "pointSet":
            cont = pointToCell(sourceName)
        elif sourceType == "cellZone":
            cont = zoneToCell(sourceName)
        elif sourceType == "faceZone":
            cont = faceZoneToCell(sourceName)
        elif sourceType == "surface":
            cont = surfaceToCell(sourceName)
        elif sourceType == "label":
            cont = labelToCell(sourceName)
        elif sourceType == "shape":
            cont = shapeToCell(sourceName)
        elif sourceType == "box":
            cont = boxToCell(sourceName)
        elif sourceType == "cylinder":
            cont = cylinderToCell(sourceName)
        elif sourceType == "sphere":
            cont = sphereToCell(sourceName)
        elif sourceType == "rotatedBox":
            cont = rotatedBoxToCell(sourceName)
        elif sourceType == "nearest":
            cont = nearestToCell(sourceName)
        elif sourceType == "field":
            cont = fieldToCell(sourceName)
        elif sourceType == "region":
            cont = regionToCell(sourceName)
    elif resultType == "faceSet":
        if   sourceType == "cellSet":
            cont = cellToFace(sourceName)
        elif sourceType == "faceSet":
            cont = faceToFace(sourceName)
        elif sourceType == "pointSet":
            cont = pointToFace(sourceName)
        elif sourceType == "faceZone":
            cont = zoneToFace(sourceName)
        elif sourceType == "label":
            cont = labelToFace(sourceName)
        elif sourceType == "box":
            cont = boxToFace(sourceName)
        elif sourceType == "patch":
            cont = patchToFace(sourceName)
        elif sourceType == "normal":
            cont = normalToFace(sourceName)
    elif resultType == "pointSet":
        if   sourceType == "cellSet":
            cont = cellToPoint(sourceName)
        elif sourceType == "faceSet":
            cont = faceToPoint(sourceName)
        elif sourceType == "pointSet":
            cont = pointToPoint(sourceName)
        elif sourceType == "cellZone":
            cont = zoneToPoint(sourceName)
        elif sourceType == "surface":
            cont = surfaceToPoint(sourceName)
        elif sourceType == "label":
            cont = labelToPoint(sourceName)
        elif sourceType == "box":
            cont = boxToPoint(sourceName)
        elif sourceType == "nearest":
            cont = nearestToPoint(sourceName)
    elif resultType == "cellZoneSet":
        if sourceType == "cellSet":
            cont = setToCellZone(sourceName)
    elif resultType == "faceZoneSet":
        if sourceType == "faceSet":
            cont = setToFaceZone(sourceName)
        elif sourceType == "sets":
            cont = setsToFaceZone(sourceName)       #sourceNameはlist形式
    else:
        cont = ""

    return cont


#  makeHeader
#    topoSetDictのheader作成
def makeHeader():
    header = """/*--------------------------------*- C++ -*----------------------------------*\\
| =========                 |                                                 |
| \\\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox           |
|  \\\    /   O peration     | Version:  2.1.0                                 |
|   \\\  /    A nd           | Web:      www.OpenFOAM.org                      |
|    \\\/     M anipulation  |                                                 |
\\*---------------------------------------------------------------------------*/
FoamFile
{
    version     2.0;
    format      ascii;
    class       dictionary;
    object      topoSetDict;
}
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //
"""
    return header

#  makeFooter
#
def makeFooter():
    footer = """// ************************************************************************* //"""
    return footer

#
#  addTopoSetDict
#    topoSetDictに項目を追加して保存
def addTopoSetDict(caseDir, cont):
    fileName = caseDir + "/system/topoSetDict"
    f=open(fileName)
    topoSetDict = f.read()
    f.close()
    p = getFoamContents.skipFoamFileMem(topoSetDict, 0)
    p = getFoamContents.getKeywordPointerMem("actions", topoSetDict, p)
    sp = p
    [actions, p] = getFoamContents.getSmallPairMem(topoSetDict, p)
    ep = p
    actions += topoSetDict[sp+1:ep-1] + cont
    newDict = topoSetDict[:sp] + "actions\n(" + actions + ")" + topoSetDict[ep:]
    f=open(fileName, "w")
    f.write(newDict)
    f.close()


#
#  saveTopoSetDict
#    topoSetDictをsystemフォルダに保存
def saveTopoSetDict(caseDir, topoSetDict):
    fileName = caseDir + "/system/topoSetDict"
    f=open(fileName, "w")
    f.write(topoSetDict)
    f.close()

#
#  runTopoSet
#    topoSetの実行
def runTopoSet():
    comm = "topoSet"
    pyTreeFoam.runOFUtility(comm)





#
#  main
#
if __name__ == "__main__":
    caseDir = sys.argv[1]
    os.chdir(caseDir)

    topoSetDict = makeHeader()
    topoSetDict += "\naction\n(\n"
    sourceName = "fineZone"
    sourceType = "cellSet"
    action = "add"
    resultName = "fineZone-1"
    resultType = "cellZone"
"""
cont = makeContents(sourceType, sourceName, action, resultType, resultName)
if cont == "":
    print "ERRER: " + sourceType + " To " + resultType + " の組み合わせはsetできません。"
else:
    addTopoSetDict(caseDir, cont)
    topoSetDict += cont + ");\n\n"
    topoSetDict += makeFooter()
    #print topoSetDict
"""

